local utils = require 'mp.utils'

---------------------------------
-- python 3 is required


-- path to jvc_control python script
JVC_CONTROL_SCRIPT_PATH = "<path-to>/jvc_control.py"

-- prefix of tag in filename to detect aspect ratio
-- AR vaue must contain only numbers (e.g. 178, 185, 240,....)
FILENAME_AR_PREFIX = "_AR"

-- delay for sending command after detected frame rate switch
-- (projector is not available for some time after switching frame rate)
JVC_FPS_SWITCH_DELAY = 1 -- in s

-- group commands for sending multiple commands at once
-- !! the groups and its commands most likely needs to be adapted to your requirements !!
-- !! please change it to fit your needs !!
-- !! e.g. use different aspect ratios, picture formats, .... !!!
commandGroup = {}
commandGroup['HDR'] 		= { 'lamp_high' } 						-- HDR mode
commandGroup['SDR'] 		= { 'lamp_low' } 						-- SDR mode
commandGroup['178'] 		= { 'mask_off', 'memory1' } 			-- aspect ratio 1.78:1, taken from filename
commandGroup['185'] 		= { 'mask_off', 'memory2' } 			-- aspect ratio 1.85:1, taken from filename
commandGroup['235'] 		= { 'mask_custom1', 'memory3' } 		-- aspect ratio 2.35:1, taken from filename
commandGroup['240'] 		= { 'mask_custom2', 'memory4' } 		-- aspect ratio 2.40:1, taken from filename
commandGroup['Default'] 	= { 'lamp_low', 'mask_off', 'memory1' } -- set to default values, after movie ends


isPlaying = false
gamma = nil
aspect = nil
commandGroups = {}

-----------------------------------------
-- will be called when the movie starts
function on_start(event)
	isPlaying = true
	sendCommands()
end

-----------------------------------------
-- will be called when the movie ends
function on_end(event)
	commandGroups = { "Default" }
	sendCommands(true)
	isPlaying = false
end

-----------------------------------------
-- will be called when the movie pauses
function on_pause(name, value)
	if isPlaying == true then
		if value == true then
			print("pause")
		else
			print("unpause")
		end
	end
end

-----------------------------------------
-- will be called when the filename can be read
function on_filename(name, value)
	if value ~= nil then
		arStart, arEnd = string.find(value, FILENAME_AR_PREFIX.."%d+")

		if arStart ~= nil then
			aspect = string.sub(value, arStart + 3, arEnd)

			print("AR", aspect)
			table.insert(commandGroups, aspect)
			sendCommands()
		end
	end
end

-----------------------------------------
-- will be called when the aspect ration can be read
function on_aspect(name, value)
	if value ~= nil then
		aspect = tostring(round2digit(value) * 100)

		print("AR", aspect)
		table.insert(commandGroups, aspect)
		sendCommands()
	end
end

-----------------------------------------
-- will be called when the gamma can be read
function on_gamma(name, value)
	if value ~= nil then
		if value == "pq" then
			gamma = "HDR"
			print("HDR")
		else
			gamma = "SDR"
			print("SDR")
		end

		table.insert(commandGroups, 1, gamma)
		sendCommands()
	end
end

-----------------------------------------
-- translate groups to single commands and sends it to the projector
function sendCommands(immediately)
	if canSend() == false then
		return
	end

	local startCommands = {"python3", JVC_CONTROL_SCRIPT_PATH}
	for _, cmdGrp in ipairs(commandGroups) do
		for _, cmd in ipairs(commandGroup[cmdGrp]) do
			table.insert(startCommands, cmd)
		end
	end

	local fpsChanged = fpsChanged()
	print("fps changed", fpsChanged)

	if fpsChanged == true and immediately ~= true then
		mp.add_timeout(JVC_FPS_SWITCH_DELAY,
			function()
				exec(startCommands)
			end
		)
	else
		exec(startCommands)
	end
end

-----------------------------------------
-- checks if all data exists for sending sending commands
function canSend()
	return isPlaying == true and gamma ~= nil and aspect ~= nil
end

-----------------------------------------
-- calls the python script
function exec(args)
	local r = mp.command_native({
        name = "subprocess",
        capture_stdout = true,
        capture_stderr = true,
        args = args,
        playback_only = false,
        detach = true
    })
    if r ~= nil and r.stdout ~= nil then
	    print(r.stdout)
	end
end

-----------------------------------------
-- checks if movie fps differs from display fps
function fpsChanged()
	local fpsDisp = round2digit(mp.get_property_number("display-fps"))
	local fpsVid = round2digit(mp.get_property_number("container-fps"))
	print("fpsDisp", fpsDisp)
	print("fpsVideo", fpsVid)
	return fpsDisp ~= fpsVid
end

function addAll(target, source)
	for _, value in ipairs(source) do
		table.insert(target, value)
	end
end

function round2digit(fps)
	if fps ~= nil then
		return math.floor(fps * 100 + 0.5) / 100
	end
	return 0
end

mp.register_event("file-loaded", on_start)
mp.register_event("end-file", on_end)
mp.observe_property("pause", "bool", on_pause)
mp.observe_property("filename", "string", on_filename)
mp.observe_property("video-params/gamma", "string", on_gamma)
--mp.observe_property("video-params/aspect", "number", on_aspect)