#!/usr/bin/python3
import sys
import socket
from enum import Enum


###########################################
#
# Script to send commands to a JVC projector
# Usage: jvc_control.py <command1> [<command2>..]
#
# Command must be one of Commands (see below)
# e.g. "jvc_command.py power_on" or "jvc_command.py hdmi1 memory1 iris_auto_1"
#
# Important notice: this script will take care of sending multiple commands in one shot.
# If script is called multiple times without a delay (0.5-1s), projector will most likely reject connection
#

# hostname / IP of projector
HOST = '<projector-IP>'


class Commands(Enum):
    # http://support.jvc.com/consumer/support/documents/DILAremoteControlGuide.pdf

    # power commands
    power_on = b"\x21\x89\x01\x50\x57\x31\x0A"
    power_off = b"\x21\x89\x01\x50\x57\x30\x0A"

    # lens memory commands
    memory1 = b"\x21\x89\x01\x49\x4E\x4D\x4C\x30\x0A"
    memory2 = b"\x21\x89\x01\x49\x4E\x4D\x4C\x31\x0A"
    memory3 = b"\x21\x89\x01\x49\x4E\x4D\x4C\x32\x0A"
    memory4 = b"\x21\x89\x01\x49\x4E\x4D\x4C\x33\x0A"
    memory5 = b"\x21\x89\x01\x49\x4E\x4D\x4C\x34\x0A"

    # input commands
    hdmi1 = b"\x21\x89\x01\x49\x50\x36\x0A"
    hdmi2 = b"\x21\x89\x01\x49\x50\x37\x0A"

    # power status query commands
    power_status   = b"\x3F\x89\x01\x50\x57\x0A"
    current_output = b"\x3F\x89\x01\x49\x50\x0A"
    
    # picture mode commands
    pm_cinema = b"\x21\x89\x01\x50\x4D\x50\x4D\x30\x31\x0A"
    pm_hdr = b"\x21\x89\x01\x50\x4D\x50\x4D\x30\x34\x0A"
    pm_natural = b"\x21\x89\x01\x50\x4D\x50\x4D\x30\x33\x0A"
    pm_film = b"\x21\x89\x01\x50\x4D\x50\x4D\x30\x30\x0A"
    pm_THX = b"\x21\x89\x01\x50\x4D\x50\x4D\x30\x36\x0A"
    pm_user1 = b"\x21\x89\x01\x50\x4D\x50\x4D\x30\x43\x0A"
    pm_user2 = b"\x21\x89\x01\x50\x4D\x50\x4D\x30\x44\x0A"
    pm_user3 = b"\x21\x89\x01\x50\x4D\x50\x4D\x30\x45\x0A"
    pm_user4 = b"\x21\x89\x01\x50\x4D\x50\x4D\x30\x46\x0A"
    pm_user5 = b"\x21\x89\x01\x50\x4D\x50\x4D\x31\x30\x0A"
    pm_user6 = b"\x21\x89\x01\x50\x4D\x50\x4D\x31\x31\x0A"
    pm_hlg = b"\x21\x89\x01\x50\x4D\x50\x4D\x31\x34\x0A"

    # low latency enable/disable
    pm_low_latency_enable = b"\x21\x89\x01\x50\x4D\x4C\x4C\x31\x0A"
    pm_low_latency_disable = b"\x21\x89\x01\x50\x4D\x4C\x4C\x30\x0A"

    # mask commands
    mask_off = b"\x21\x89\x01\x49\x53\x4D\x41\x32\x0A"
    mask_custom1 = b"\x21\x89\x01\x49\x53\x4D\x41\x30\x0A"
    mask_custom2 = b"\x21\x89\x01\x49\x53\x4D\x41\x31\x0A"
    mask_custom3 = b"\x21\x89\x01\x49\x53\x4D\x41\x33\x0A"

    #lamp commands
    lamp_high = b"\x21\x89\x01\x50\x4D\x4C\x50\x31\x0A"
    lamp_low = b"\x21\x89\x01\x50\x4D\x4C\x50\x30\x0A"

    #menu controls
    menu = b"\x21\x89\x01\x52\x43\x37\x33\x32\x45\x0A"
    menu_down = b"\x21\x89\x01\x52\x43\x37\x33\x30\x32\x0A"
    menu_left = b"\x21\x89\x01\x52\x43\x37\x33\x33\x36\x0A"
    menu_right = b"\x21\x89\x01\x52\x43\x37\x33\x33\x34\x0A"
    menu_up = b"\x21\x89\x01\x52\x43\x37\x33\x30\x31\x0A"
    menu_ok = b"\x21\x89\x01\x52\x43\x37\x33\x32\x46\x0A"
    menu_back = b"\x21\x89\x01\x52\x43\x37\x33\x30\x33\x0A"

    #Lens Aperture commands
    aperture_off = b"\x21\x89\x01\x50\x4D\x44\x49\x30\x0a"
    aperture_auto1 = b"\x21\x89\x01\x50\x4D\x44\x49\x31\x0a"
    aperture_auto2 = b"\x21\x89\x01\x50\x4D\x44\x49\x32\x0a"

    #Anamorphic commands
    anamorphic_off = b"\x21\x89\x01\x49\x4E\x56\x53\x30\x0A"
    anamorphic_a = b"\x21\x89\x01\x49\x4E\x56\x53\x31\x0A"
    anamorphic_b = b"\x21\x89\x01\x49\x4E\x56\x53\x32\x0A"
    anamorphic_c = b"\x21\x89\x01\x49\x4E\x56\x53\x33\x0A"

    # Iris commands
    iris_manual = b"\x21\x89\x01\x50\x4D\x44\x49\x30\x0a" # IRIS MANUAL
    iris_auto_1 = b"\x21\x89\x01\x50\x4D\x44\x49\x31\x0a" # IRIS Auto 2
    iris_auto_2 = b"\x21\x89\x01\x50\x4D\x44\x49\x32\x0a" # IRIS Auto 2
    iris_0 = "\x21\x89\x01\x50\x4D\x4C\x41\x30\x30\x30\x30\x0a" # IRIS 0
    iris_m1 = b"\x21\x89\x01\x50\x4D\x4C\x41\x2D\x30\x30\x31\x0a" # IRIS -1
    iris_m2 = b"\x21\x89\x01\x50\x4D\x4C\x41\x2D\x30\x30\x32\x0a" # IRIS -2
    iris_m3 = b"\x21\x89\x01\x50\x4D\x4C\x41\x2D\x30\x30\x33\x0a" # IRIS -3
    iris_m4 = b"\x21\x89\x01\x50\x4D\x4C\x41\x2D\x30\x30\x34\x0a" # IRIS -4
    iris_m5 = b"\x21\x89\x01\x50\x4D\x4C\x41\x2D\x30\x30\x35\x0a" # IRIS -5
    iris_m6 = b"\x21\x89\x01\x50\x4D\x4C\x41\x2D\x30\x30\x36\x0a" # IRIS -6
    iris_m7 = b"\x21\x89\x01\x50\x4D\x4C\x41\x2D\x30\x30\x37\x0a" # IRIS -7
    iris_m8 = b"\x21\x89\x01\x50\x4D\x4C\x41\x2D\x30\x30\x38\x0a" # IRIS -8
    iris_m9 = b"\x21\x89\x01\x50\x4D\x4C\x41\x2D\x30\x30\x39\x0a" # IRIS -9


class PowerStates(Enum):
    standby   = "30"
    lamp_on   = "31"
    cooling   = "32"
    reserved  = "33"
    emergency = "34"


class JVCProjector:
    """JVC Projector Control"""


    def __init__(self, host, port=20554, connect_timeout=1):
        self.host = host
        self.port = port
        self.connect_timeout = connect_timeout
        self.BUFFER_SIZE = 1024

    def connect(self):
        JVC_GREETING = b'PJ_OK'
        JVC_REQ = b'PJREQ'
        JVC_ACK = b'PJACK'

        self.jvc_sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.jvc_sock.settimeout(self.connect_timeout)
        self.jvc_sock.connect((self.host, self.port)) # connect to projector

        # 3 step handshake:
        # Projector sends PJ_OK, client sends PJREQ, projector replies with PJACK
        # first, after connecting, see if we receive PJ_OK. If not, raise exception
        if self.jvc_sock.recv(self.BUFFER_SIZE) != JVC_GREETING:
            raise Exception("Projector did not reply with correct PJ_OK greeting")

        # try sending PJREQ, if there's an error, raise exception
        try:
            self.jvc_sock.send(JVC_REQ)
        except socket.error as e:
            raise Exception("Socket exception when sending PJREQ")

        # see if we receive PJACK, if not, raise exception
        if self.jvc_sock.recv(self.BUFFER_SIZE) != JVC_ACK:
            raise Exception("Socket exception on PJACK")

    def disconnect(self):
        self.jvc_sock.close()
        self.jvc_sock = None

    def _send_command(self, operation, ack=None):
        result = False

        self.jvc_sock.send(operation)

        result = self.jvc_sock.recv(self.BUFFER_SIZE).hex()
        success = self.checkResult(operation.hex(), result)

        if ack and success:
            split = len(operation.hex())
            data = result[split:]
            code = data[10:12]
            if len(code) > 0:
                print(PowerStates(code).name)
                return True
            else:
                raise Exception("No ACK data")

        print("%s"%success)

        return success

    def command(self, command_string):
        if not hasattr(Commands, command_string):
            print("invalid")
            return False
        else:
            if (command_string == 'power_status'):
                self._send_command(Commands[command_string].value, True)
            else:
                self._send_command(Commands[command_string].value)
            return True

    def checkResult(self, send, result):
        if result != None:
            expected = send[6:10]
            received = result[6:10]

            return expected == received

        return False


def sendCommands(cmds):
	projector = JVCProjector(HOST)
	projector.connect()

	for cmd in cmds:
		print(cmd, end = ": ")
		try:
			projector.command(cmd)
		except:
			print("error")

	projector.disconnect()


numArgs = len(sys.argv)
if numArgs > 1:
	reqCmds = []
	for idx in range(1, numArgs):
		reqCmds.append(sys.argv[idx])

	sendCommands(reqCmds)

else:
	print("Usage: jvc_control.py <command1> [<command2>..]")

